/*
 *       ____   ____      
 *    ___\   \ /   /____  
 *  _/ __ \   Y   // __ \ 
 *  \  ___/\     /\  ___/ 
 *   \___  >\___/  \___  >
 *       \/            \/ 
 *
 *  YaMoR simulator
 *
 *  Cyril Jaquier    
 *  Kevin Drapel
 *
 *  BIRG / LSL - EPFL
 *  http://birg.epfl.ch
 *
 * $Id$
 *
 */
 

#include "system.h"
#include "mmgr.h"  

#include "button.h"

/*    
 */

namespace eve {
	
Button::Button() {
	m_texture = NULL; 
}

Button::~Button() {
}

/**
 * Setup a button with icon
 */
void Button::setup(std::string iconFilename, rect<s32> dest) {
	m_texture = System::getInstance()->getVideoDriver()->getTexture(iconFilename.c_str());
	m_rect = dest; 
}

/**
 * Check if mouse is over the button
 */
bool Button::isMouseOver(int mouseX, int mouseY) {
	return m_rect.isPointInside(position2d<s32>(mouseX, mouseY)); 
}

/**
 * Draw button via Irrlicht
 */
void Button::draw(IVideoDriver *driver, int mouseX, int mouseY, bool isOn) {
	SColor s[4]; 
	float scale = 1.0; 
	
	if (isMouseOver(mouseX, mouseY)) {
		s[0] = SColor(255,255,255,255);
		s[1] = SColor(255,255,255,255);
		s[2] = SColor(255,255,255,255);
		s[3] = SColor(255,255,255,255);
	} else {
		if (!isOn) {
			s[0] = SColor(200,200,200,200);
			s[1] = SColor(200,200,200,200);
			s[2] = SColor(200,200,200,200);
			s[3] = SColor(200,200,200,200);
		} else {
			scale = 0.5+0.5*fabs(sin(System::getInstance()->getTimer()->T*2.0)); 
			int blink = (int) sin(System::getInstance()->getTimer()->T*4.0)*40; 
			s[0] = SColor(200+blink,200+blink,200+blink,200+blink);
			s[1] = SColor(200+blink,200+blink,200+blink,200+blink);
			s[2] = SColor(200+blink,200+blink,200+blink,200+blink);
			s[3] = SColor(200+blink,200+blink,200+blink,200+blink);
		}
	}
	
	 
	position2d<s32> p = m_rect.getCenter(); 
	position2d<s32> p1 = p;
	position2d<s32> p2 = p; 
	int h = m_rect.getHeight()>>1; 
	int w = m_rect.getWidth()>>1; 
	h = (int) (h*scale);
	w = (int) (w*scale); 
	position2d<s32> rc(w,h); 
	rect<s32> tempRect(p-rc, p+rc); 
	System::getInstance()->getVideoDriver()->draw2DImage(m_texture, 
						tempRect, rect<s32>(0,0,m_texture->getSize().Width,
						m_texture->getSize().Height), 0, (SColor*) &s, true);
}
			

			
			

} // 
