/*
 *       ____   ____      
 *    ___\   \ /   /____  
 *  _/ __ \   Y   // __ \ 
 *  \  ___/\     /\  ___/ 
 *   \___  >\___/  \___  >
 *       \/            \/ 
 *
 *  YaMoR simulator
 *
 *  Cyril Jaquier    
 *  Kevin Drapel
 *
 *  BIRG / LSL - EPFL
 *  http://birg.epfl.ch
 *
 * $Id$
 *
 */
 

#include "system.h"
#include "mmgr.h"
#include "utils.h"

#include "camera.h"

/*    
 */

namespace eve {
	
	
Camera::Camera() {
	m_angleX = 0.0; 
	m_angleY = 0.0;
	m_angleOldX = 0.0; 
	m_angleOldY = 0.0;
	rotate(0,0); 
	m_isDragging = false;
}

Camera::~Camera() {
}


/**
 * Move the camera forward or backward
 */
void Camera::move(double amount) {
	m_position += m_direction*amount; 
}

/**
 * Strafe left right
 */
void Camera::strafe(double amount) {
	m_position += m_right*amount;
}

/**
 * Move up down
 */
void Camera::verticalStrafe(double amount) {
	m_position += m_up*amount;
}

/**
 * Compute new angles when the mouse is pressed
 */
void Camera::mouseDown(int x, int y) {

	if (m_isDragging) {
	
		m_angleX = m_angleOldX + (double)(x-m_startX)*0.005;
		m_angleY = m_angleOldY + (double)(y-m_startY)*0.005;
		
		rotate(m_angleX, m_angleY);
		return; 
	}
	
	m_startX = x;
	m_startY = y;
	
	m_isDragging = true;
}

/**
 * Keep the old angles when the mouse is not pressed anymore
 */
void Camera::mouseUp() {
	m_isDragging = false;
	
	m_angleOldX = m_angleX; 
	m_angleOldY = m_angleY; 
}

/**
 * Rotate the FPS-like camera
 */
void Camera::rotate(double angleX, double angleY) {
	
	m_direction.X = cos(m_angleX)*cos(m_angleY); 
	m_direction.Z = sin(m_angleX)*cos(m_angleY); 
	m_direction.Y = sin(m_angleY);
	
	m_right.X = cos(m_angleX-3.14/2.0); 
	m_right.Y = 0.0;
	m_right.Z = sin(m_angleX-3.14/2.0); 
	
	m_direction.normalize();
	m_right.normalize(); 
	
	m_up = m_direction.crossProduct(m_right); 
}


} // 
