/*
 *       ____   ____      
 *    ___\   \ /   /____  
 *  _/ __ \   Y   // __ \ 
 *  \  ___/\     /\  ___/ 
 *   \___  >\___/  \___  >
 *       \/            \/ 
 *
 *  YaMoR simulator
 *
 *  Cyril Jaquier    
 *  Kevin Drapel
 *
 *  BIRG / LSL - EPFL
 *  http://birg.epfl.ch
 *
 * $Id$
 *
 */
 

#include "system.h"
#include "mmgr.h"  

#include "yamor.h"
#include "mesh.h"
#include "graph.h"
#include "graphnode.h"

/*    
 */

namespace eve {
	
Graph::Graph() {
	m_traversalOrder.clear(); 
}

Graph::~Graph() {
	for (int j=0;j<m_nodes.size();j++) {
		delete m_nodes[j];
	}
}

/**
 * Create a graph node
 */
void Graph::createNode(int address) {
	GraphNode *g = new GraphNode(address); 
	m_nodes.push_back(g); 		
}

/**
 * Get node from its Bluetooth address
 */
GraphNode *Graph::getNodeFromAddress(int address) {
	for (int j=0;j<m_nodes.size();j++) {
		if (m_nodes[j]->m_address == address) return m_nodes[j];
	}
	return NULL;
}

/**
 * Connect two nodes
 */
void Graph::createConnection(GraphNode *node1, GraphNode *node2) {
	node1->m_children.push_back(node2); 
	node2->m_children.push_back(node1); 
}

/**
 * Parse the graph and store the nodes with respect to their level
 */
void Graph::scan(GraphNode *parent, GraphNode *node) {
	m_traversalOrder.push_back(node->m_address); 
	printf("%d\n", node->m_address); 
	
	for (int n=0;n<node->m_children.size();n++) {
		if (node->m_children[n]!=parent) {
			scan(node,node->m_children[n]);
		}
	}
	
	node->m_parent = parent;	
}


} // 
