/*
 *       ____   ____      
 *    ___\   \ /   /____  
 *  _/ __ \   Y   // __ \ 
 *  \  ___/\     /\  ___/ 
 *   \___  >\___/  \___  >
 *       \/            \/ 
 *
 *  YaMoR simulator
 *
 *  Cyril Jaquier    
 *  Kevin Drapel
 *
 *  BIRG / LSL - EPFL
 *  http://birg.epfl.ch
 *
 * $Id$
 *
 */
 
#ifndef _MESH_H_
#define _MESH_H_

#include <ode/ode.h>
#include <irrlicht.h>
#include <string>

using namespace irr;
using namespace core;
using namespace scene;
using namespace video; 

namespace eve {

	// external reference
	class OdeWrapper; 

	// class definition
	class Mesh {
		public:
			Mesh();
			~Mesh(); 
			void loadMesh(std::string filename); 
			void addMesh(); 

			void createBoxGeomFromMesh(vector3df translate = vector3df(0,0,0));
			void createBodyFromMesh(double mass, vector3df centerOfMass, bool isMovable);
			void updateMeshAfterPhysics();
		
			dGeomID getMainGeomID() { return m_geomID; }
			dGeomID getEncapsulatedGeomID() { return m_meshGeomID; }
			dBodyID getBodyID() { return m_bodyID; }
			
			IAnimatedMeshSceneNode *getNode() { return m_node; }
			double getMass() { return m_mass; }
			
			vector<triangle3df> extractTriangles(IAnimatedMeshSceneNode *sceneNode);
			dGeomID createGeom(double quakeScale, vector<triangle3df> t3df);
			
		protected:
			IAnimatedMeshSceneNode *m_node;

			dGeomID		m_geomID; 		
			dGeomID		m_meshGeomID; 
			dBodyID		m_bodyID;
			
			IAnimatedMesh *m_mesh;
			
			double		m_mass;
			vector3df 	m_centerOfMass;
			
			dVector3 *m_vertices;
			int *m_indices;
	};	
	
	
} // namespace


#endif
