/*
 *       ____   ____      
 *    ___\   \ /   /____  
 *  _/ __ \   Y   // __ \ 
 *  \  ___/\     /\  ___/ 
 *   \___  >\___/  \___  >
 *       \/            \/ 
 *
 *  YaMoR simulator
 *
 *  Cyril Jaquier    
 *  Kevin Drapel
 *
 *  BIRG / LSL - EPFL
 *  http://birg.epfl.ch
 *
 * $Id$
 *
 */
 
#ifndef _ODEWRAPPER_H_
#define _ODEWRAPPER_H_

#include <ode/ode.h>
#include <string>

namespace eve {

	class OdeGlobalParameters {
		public: 
			OdeGlobalParameters() { 
				cfm = 0.0; 						// cfm at 0.0 => joints can't move
				erp = 0.3; 						// error correction (0.1 to 0.9)
				gravity = -100; 				// a reasonable gravity on Y axis
				useQuickStep = true; 			// quickstep is used by default
				quickStepIterations = 20; 		// 20 is the default value
				contactSurfaceLayer = 0.0;		// increasing (0.001) may reduce jitter with contacts
				contactMaxVelocity = dInfinity;	// decreasing may reduce explosion with contacts
				maxContacts = 100;
				
				step = 1.0/700.0;					// 1.0/steps = integration step
				update = 1.0/300.0;					// 1.0/substeps = time for one physics frame
				guiSpeed = 100;
				
				surfaceFriction = 1e-5;
				surfaceBounce = 0.0;
			}
			
			~OdeGlobalParameters() {}
			
		public: 
			double cfm;
			double erp;
			double gravity; 
			bool   useQuickStep;
			unsigned int quickStepIterations; 
			double contactSurfaceLayer; 
			double contactMaxVelocity; 
			int maxContacts;
			
			double step;
			double update;
			double guiSpeed;
			
			double surfaceFriction;
			double surfaceBounce;
	};
	
	// SHOULD BE A SINGLETON
	class OdeWrapper {
		public:
			 OdeWrapper();
			 ~OdeWrapper(); 
			 void init(OdeGlobalParameters &odeGlobalParameters);
			 void dispose();
			
			 static void nearCollisionCallback(void* data, dGeomID o1, dGeomID o2);
			
			 dWorldID getWorldID() { return m_world; } 
			 dSpaceID getSpaceID() { return m_space; }
			 dJointGroupID getContactGroupID() { return m_contactGroup; }
			 dJointGroupID getJointGroupID() { return m_jointGroup; }
			 dJointGroupID getFixedJointGroupID() { return m_fixedJointGroup; }
			 dGeomID getTerrainGeom() { return m_terrainGeom; }
			 
			 dJointID addFixed(dBodyID body1, dBodyID body2);
			 void step(double stepSize);
			 void setTerrainGeom(dGeomID terrainGeom) { m_terrainGeom = terrainGeom;}
			 
			 static void setCurrentWrapper(OdeWrapper *current) { ptrToCurrent = current; }
			 
		protected:
			dWorldID m_world;
			dSpaceID m_space; 
			dJointGroupID m_contactGroup; 
			dJointGroupID m_jointGroup; 
			dJointGroupID m_fixedJointGroup; 
			
			static OdeGlobalParameters m_globalParameters; 
			
			// to remove...
			dGeomID	m_terrainGeom; 
			
			static OdeWrapper *ptrToCurrent; 
			
			static dContact *m_contact; 
	};


	
} // namespace

#endif
