/*
 *       ____   ____      
 *    ___\   \ /   /____  
 *  _/ __ \   Y   // __ \ 
 *  \  ___/\     /\  ___/ 
 *   \___  >\___/  \___  >
 *       \/            \/ 
 *
 *  YaMoR simulator
 *
 *  Cyril Jaquier    
 *  Kevin Drapel
 *
 *  BIRG / LSL - EPFL
 *  http://birg.epfl.ch
 *
 * $Id$
 *
 */
 
#ifndef _ROBOT_H_
#define _ROBOT_H_

#include <string>
#include <vector>

#include <ode/ode.h>
#include <irrlicht.h>

using namespace std;
using namespace irr;

namespace eve {

	typedef struct {
		int address1;
		int address2;
		YAMOR_CONNECTOR co1;
		YAMOR_CONNECTOR co2;
		double angle;
	} Connection;
	
	class Graph; 
	
	// class definition
	class Robot {
		public:
			Robot(double massModule, double massLeveler);
			~Robot(); 
			void addModule(int address, std::string bluetoothAddr, vector3df position = vector3df(0.0,0.0,0.0), vector3df rotation = vector3df(0.0, 0.0, 0.0)); 
			void addConnection(int address1, int address2, YAMOR_CONNECTOR co1, YAMOR_CONNECTOR co2, double angle); 
			void alignModules(int addressParent, int addressChild, YAMOR_CONNECTOR co1, YAMOR_CONNECTOR co2, double angle);
			void fixModules(int addressParent, int addressChild, YAMOR_CONNECTOR co1, YAMOR_CONNECTOR co2);
			void generateODEData(); 
			void updatePhysics();
			void setPosition(int address, double position, bool invert); 
			void tilt(float amount);
			void connect(); 
			
			aabbox3d<f32> getBoundingBox(double scaleRatio=1.0);
			
			Yamor *getModule(int address);
			Yamor *getModule(std::string bluetoothAddr); 
			void isMovable(bool isMovable, int fixedModuleAddress);
			
			int getTotalModules() { return m_modules.size(); }
			
		public:
			vector<Yamor*> m_modules;
			vector<Connection> m_connections;
			
			bool m_isMovable;
			Yamor *m_fixedModule;
			
		private:
			Graph *generateSceneGraph();
			int isParentInConnection(int id1, int id2, vector<int> traversalOrder);
			double m_moduleMass;
			double m_levelerMass; 
 	};	
	
	
} // namespace


#endif
