/*
 *       ____   ____      
 *    ___\   \ /   /____  
 *  _/ __ \   Y   // __ \ 
 *  \  ___/\     /\  ___/ 
 *   \___  >\___/  \___  >
 *       \/            \/ 
 *
 *  YaMoR simulator
 *
 *  Cyril Jaquier    
 *  Kevin Drapel
 *
 *  BIRG / LSL - EPFL
 *  http://birg.epfl.ch
 *
 * $Id$
 *
 */
 
#ifndef _GLOBALS_H_
#define _GLOBALS_H_

#include <ode/ode.h>
#include <irrlicht.h>
#include <string>
#include <assert.h>

#include "timer.h"
#include "odewrapper.h"
#include "camera.h"


using namespace irr;
using namespace core;
using namespace scene;
using namespace video;
using namespace io;
using namespace gui;

#include "meshmanager.h"

namespace eve {

	// class definition
	class System {
		public:
			static System* getInstance(); 
			static void deleteInstance(); 
			
			IrrlichtDevice *getDevice() { return m_irrlichtDevice; }
			IVideoDriver *getVideoDriver() { return m_irrlichtVideoDriver; }
			ISceneManager *getSceneManager() { return m_irrlichtSceneManager; }
			ICameraSceneNode *getCameraIrrlicht() { return m_irrlichtCameraSceneNode; }
			Timer *getTimer() { return m_timer; }
			OdeWrapper *getODEWrapper() { return m_odeWrapper; }
			MeshManager *getMeshManager() { return m_meshManager; }
			Camera *getCamera() { return m_camera; }
			
			int initIrrlicht(int screenWidth, int screenHeight, bool fullscreen=false, IEventReceiver *receiver=0);
			int initODE(OdeGlobalParameters &odeGlobalParameters);
			
			void disposeIrrlicht(); 
			void disposeODE(); 
			
			void updateTitleBar();
			void updateCamera();
	
		protected:
			System();
			~System(); 

		private:
			static System* m_instance; 
			
			IrrlichtDevice *m_irrlichtDevice;
			IVideoDriver *m_irrlichtVideoDriver;
			ISceneManager *m_irrlichtSceneManager;
			ICameraSceneNode *m_irrlichtCameraSceneNode;
			
			Timer *m_timer; 
			Camera *m_camera;
			OdeWrapper *m_odeWrapper;
			MeshManager *m_meshManager; 
	};	
	
} // namespace

#endif


