/*
 *       ____   ____      
 *    ___\   \ /   /____  
 *  _/ __ \   Y   // __ \ 
 *  \  ___/\     /\  ___/ 
 *   \___  >\___/  \___  >
 *       \/            \/ 
 *
 *  YaMoR simulator
 *
 *  Cyril Jaquier    
 *  Kevin Drapel
 *
 *  BIRG / LSL - EPFL
 *  http://birg.epfl.ch
 *
 * $Id$
 *
 */
 
  
#include <math.h>

#include "utils.h"
#include "mmgr.h"  

namespace eve {
	
/**
 * Convert a quaternion to Euler angles
 */
void Utils::QuaternionToEuler(const dQuaternion quaternion,irr::core::vector3df &euler) {
	dReal w,x,y,z;
	w=quaternion[0];
	x=quaternion[1];
	y=quaternion[2];
	z=quaternion[3];
	double sqw = w*w;    
	double sqx = x*x;    
	double sqy = y*y;    
	double sqz = z*z; 
	// Z X Y
	euler.Z = (irr::f32) (atan2(2.0 * (x*y + z*w),(sqx - sqy - sqz + sqw))*irr::core::GRAD_PI);
	euler.X = (irr::f32) (atan2(2.0 * (y*z + x*w),(-sqx - sqy + sqz + sqw))*irr::core::GRAD_PI);  
	euler.Y = (irr::f32) (asin(-2.0 * (x*z - y*w))*irr::core::GRAD_PI);
}

/**
 * Convert Euler angles to a quaternion
 */
void Utils::EulerToQuaternion(const irr::core::vector3df &euler, dQuaternion quaternion) {
	double _heading=euler.Z*irr::core::GRAD_PI2/2.0;
	double _attitude=euler.Y*irr::core::GRAD_PI2/2.0;
	double _bank=euler.X*irr::core::GRAD_PI2/2.0;
	double c1 = cos(_heading);    
	double s1 = sin(_heading);    
	double c2 = cos(_attitude);    
	double s2 = sin(_attitude);    
	double c3 = cos(_bank);    
	double s3 = sin(_bank);
	double c1c2 = c1*c2;    
	double s1s2 = s1*s2;   
	
	quaternion[0]=(dReal) (c1c2*c3 + s1s2*s3);
	quaternion[1]=(dReal) (c1c2*s3 - s1s2*c3);
	quaternion[2]=(dReal) (c1*s2*c3 + s1*c2*s3);	
	quaternion[3]=(dReal) (s1*c2*c3 - c1*s2*s3);
}

/**
 * Tells whether a point is on the same side of another point
 */
bool Utils::isSameSide(vector3df p1, vector3df p2, vector3df pA, vector3df pB) {
	vector3df cross1 = (pB-pA).crossProduct(p1-pA); 
	vector3df cross2 = (pB-pA).crossProduct(p2-pA); 

	if (cross1.dotProduct(cross2) >= 0.0) return true; 
	return false; 
}

/**
 * Tells whether a point is inside a triangle
 */
bool Utils::isPointInsideTriangle(vector3df myPoint, vector3df pA, vector3df pB, vector3df pC) {
	if (isSameSide(myPoint, pA, pB, pC) && isSameSide(myPoint, pB, pA, pC) && isSameSide(myPoint, pC, pA, pB)) return true;
	return false; 
}

/**
 * Get distance from a point to a line
 */
float Utils::getDistanceToLine(vector3df start, vector3df end, vector3df p) {
	
	vector3df v = end-start;
	vector3df transPoint = p - start;
	
	float d = v.dotProduct(transPoint); 
	transPoint -= v*d;
	
	return transPoint.getLength(); 
}

/**
 * Print the three componets of a vector
 */
void Utils::DebugVector3df(irr::core::vector3df vec, char *name) {
	printf("%s : %f,%f,%f\n", name, vec.X, vec.Y, vec.Z); 
}

} // namespace
